package com.docus.server.common;

import com.docus.infrastructure.core.db.Sort;
import org.springframework.transaction.annotation.Transactional;

import java.io.Serializable;
import java.util.Collection;
import java.util.List;

public abstract class AbstractBaseService<T> implements IService<T> {

    protected abstract IDao<T> getDao();

    @Override
    public T findById(Serializable id) {
        return getDao().findById(id);
    }

    @Override
    public List<T> findByIds(Collection<Serializable> ids) {
        return getDao().findByIds(ids);
    }

    @Override
    public List<T> findBy(String propertyName, Object propertyValue) {
        return getDao().findBy(propertyName, propertyValue);
    }

    @Override
    public List<T> findByList(String propertyName, Collection<?> propertyValue) {
        return getDao().findByList(propertyName, propertyValue);
    }

    @Override
    public List<T> findActiveBy(String propertyName, Object propertyValue) {
        return getDao().findActiveBy(propertyName, propertyValue);
    }

    @Override
    public List<T> findBy(String propertyName, Object propertyValue, Sort sort) {
        return getDao().findBy(propertyName, propertyValue, sort);
    }

    @Override
    public List<T> findActiveBy(String propertyName, Object propertyValue, Sort sort) {
        return getDao().findActiveBy(propertyName, propertyValue, sort);
    }

    @Override
    public T findOneBy(String propertyName, Object propertyValue) {
        return getDao().findOneBy(propertyName, propertyValue);
    }

    @Override
    public T findOneByList(String propertyName, Collection<?> propertyValue) {
        return getDao().findOneByList(propertyName, propertyValue);
    }

    @Override
    public T findActiveOneBy(String propertyName, Object propertyValue) {
        return getDao().findActiveOneBy(propertyName, propertyValue);
    }

    @Override
    public List<T> findAll() {
        return getDao().findAll();
    }

    @Override
    public List<T> findAllActive() {
        return getDao().findAllActive();
    }

    @Override
    public List<T> findAll(Sort sort) {
        return getDao().findAll(sort);
    }

    @Override
    public List<T> findAllActive(Sort sort) {
        return getDao().findAllActive(sort);
    }

    @Override
    public int deleteByIdList(List<Serializable> idList) {
        return getDao().deleteByIdList(idList);
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public boolean saveBatch(Collection<T> entityList, int batchSize) {
        return getDao().saveBatch(entityList, batchSize);
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public boolean saveOrUpdate(T entity) {
        return getDao().saveOrUpdate(entity);
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public boolean updateBatchById(Collection<T> entityList, int batchSize) {
        return getDao().updateBatchById(entityList, batchSize);
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public boolean saveOrUpdateBatch(Collection<T> entityList, int batchSize) {
        return getDao().saveOrUpdateBatch(entityList, batchSize);
    }
}
